/*
 *   CDplayer 1.0 - command line Audio CDplayer
 *   Copyright (C) 1993,1994 Mark Buckaway (mark@datasoft.com)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/file.h>
#include <sys/types.h>
#include <fcntl.h>
#include <mntent.h>
#include <string.h>
#include <errno.h>
#include <linux/cdrom.h>
#include "config.h"

/* 
 * Command index numbers. These *MUST* correspond to the keywords below
 */
enum CMDS {CMD_PLAY=0,CMD_STOP,CMD_INFO,CMD_PAUSE,CMD_RESUME,CMD_EJECT};

/*
 * List of possible commands that can be issued on the command line.
 * This list is NULL terminated...nasty things will happen if the NULL is left
 * out.
 */
static char *keywords[]={"play","stop","info","pause","resume","eject",NULL};

/*
 * NEC CDROM's do strange things so we redefine the ioctl command to use
 * the special nec_ioctl function that came from xcdplayer
 */
#ifdef NEC_CDROM
#define ioctl nec_ioctl
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

void usage(char *arg0)
{
	int i;

	fputs("CDplayer 1.0 by Mark Buckaway (mark@datasoft.com)\n",stderr);
        fputs("(C) 1993,1994 DataSoft Communications\n",stderr);	  
	fprintf(stderr,"Usage: %s command [track no]\n",arg0);
	fputs("Available commands: ",stderr);
	for (i=0;keywords[i]!=NULL;i++)	 
		fprintf(stderr,"%s ",keywords[i]);
#ifdef NEC_CDROM
	fprintf(stderr,"\nCompiled to open: %s (NEC)\n\n",DEV_CDROM);
#else
	fprintf(stderr,"\nCompiled to open: %s\n\n",DEV_CDROM);
#endif
	exit(1);
}

int main(int argc, char *argv[])
{
     int i,f;
     struct cdrom_tochdr tochdr;
     struct cdrom_tocentry tocentry;
     struct cdrom_ti ti;
     struct mntent *mnt;
     FILE *fp;

/*
 * Check if a command was given. Unlike most UNIX commands, we tell the user
 * the available options if no command line options are given.
 */
     if (argc<2)
	usage(argv[0]);
     /* 
      * Check if CDROM device is already mounted, and give an error if it is.
      * This routine checks the mtab for any iso9660 file system, and then compares
      * the name of the device for that file system to the device we want to you.
      * If a match is found, the device is assumed to be already mounted.
      */
     
     if ((fp=setmntent(MOUNTED,"r"))==NULL) {
	  fprintf(stderr,"Couldn't open %s: %s\n",MOUNTED,strerror(errno));
	  exit(1);
     }
     while ((mnt=getmntent(fp))!=NULL) {
	  if (strcmp(mnt->mnt_type,"iso9660")==0) {
               if (strcmp(mnt->mnt_fsname,DEV_CDROM)!=0) continue;
	       fprintf(stderr,"CDROM \"%s\" appears to be already mounted. Unmount CDROM first.\n",DEV_CDROM);
	       endmntent(fp);
	       exit(1);
	  }
     }
     endmntent(fp);
     /*
      * CDROM device appears to be available. Open the device readonly
      */
     if ((f=open(DEV_CDROM,O_RDONLY))==-1) {
	  perror("error opening cdrom");
	  exit(1);
     }
     /* 
      * Get table of content entries for min and max tracks and 
      * store these for future reference
      */
     if ((ioctl(f,CDROMREADTOCHDR,&tochdr))==-1) {
	  perror("ioctl(CDROMREADTOHDR)");
	  exit(1);
     }
     /*
      * CDROM device is open, let's do something with it. Check
      * argument list for a command
      */
     for (i=0;keywords[i]!=NULL;i++) 
	  if ((strcmp(argv[1],keywords[i]))==0) break;
     /*
      * Do the requested operation
      */
     switch (i) {
	  case CMD_PLAY:
	       /*
		* Check if we have a requested track to play, and play it,
                * otherwise, play starting at track 1
                */
	       if (argc==3) {
		    i=atoi(argv[2]);
		    if ((i<tochdr.cdth_trk0) || (i>tochdr.cdth_trk1)) {
			 fprintf(stderr,"Track %d is invalid. Playing from track 1.\n",i);
			 ti.cdti_trk0=tochdr.cdth_trk0;
		       } else {
			 ti.cdti_trk0=i;
		    }
		  } else {
		    ti.cdti_trk0=tochdr.cdth_trk0;
	       }
	       ti.cdti_ind0=1;
	       ti.cdti_trk1=tochdr.cdth_trk1;
	       ti.cdti_ind1=1;
	       if ((ioctl(f,CDROMPLAYTRKIND,&ti))==-1) {
		    perror("ioctl(CDROMPLAYTRKIND)");
		    exit(1);
	       }
	       printf("Audio CD playing. (Start track %d)\n",ti.cdti_trk0);
	       break;
	  case CMD_STOP:
          case CMD_EJECT:
               /*
                * Stop the CD from playing! We must pause the CDROM drive
                * and then stop it otherwise some drives complain
                */
	       if ((ioctl(f,CDROMPAUSE))==-1) {
		    perror("ioctl(CDROMPAUSE)");
		    exit(1);
	       }
	       if ((ioctl(f,CDROMSTOP))==-1) {
		    perror("ioctl(CDROMSTOP)");
		    exit(1);
	       }
	       puts("Audio CD stopped.");
               if (i==CMD_EJECT) {
	          if ((ioctl(f,CDROMEJECT))==-1) {
		       perror("ioctl(CDROMEJECT)");
		       exit(1);
	          }
	       }  
	       break;
	   case CMD_INFO:
               /*
                * This is a useless command but it demostrates how to read the
                * track info from the CD. It might also be used for testing
                * or perhaps later on to figure out the Title and Track names
                * from a database...like Workman does it.
                */
               printf("Audio CD (useless) Information:\n");
	       printf("Total Tracks: %u\n",tochdr.cdth_trk1);
	       tocentry.cdte_track=CDROM_LEADOUT;
	       tocentry.cdte_format=CDROM_MSF;
	       if ((ioctl(f,CDROMREADTOCENTRY,&tocentry))==-1) {
		    perror("ioctl(CDROMREADTOCENTRY)");
		    exit(1);
	       }
	       printf("Total Playing Time: %u min %u sec\n",
                     tocentry.cdte_addr.msf.minute,
                     tocentry.cdte_addr.msf.second);
               printf("Track  Time\n");
	       for (i=tochdr.cdth_trk0;i<=tochdr.cdth_trk1;i++) {
		    tocentry.cdte_track=i;
		    tocentry.cdte_format=CDROM_MSF;
		    if ((ioctl(f,CDROMREADTOCENTRY,&tocentry))==-1) {
			 perror("ioctl(CDROMREADTOCENTRY)");
			 exit(1);
		    }
		    printf("%2d    %2u min %2u sec\n",i,tocentry.cdte_addr.msf.minute,tocentry.cdte_addr.msf.second);
	       }
	       break;
	    case CMD_PAUSE:
	       if ((ioctl(f,CDROMPAUSE))==-1) {
		    perror("ioctl(CDROMPAUSE)");
		    exit(1);
	       }
	       printf("Audio CD paused. Use \"%s resume\" to continue.\n",argv[0]);
               break;
            case CMD_RESUME:
	       if ((ioctl(f,CDROMRESUME))==-1) {
		    perror("ioctl(CDROMRESUME)");
                    fputs("Is Audio CD really paused?\n",stderr);
		    exit(1);
	       }
               puts("Audio CD resumed.");
	       break;
	  default:
               usage(argv[0]);
     }
     close(f);
     return(0);
}
